var _system = require('system'),
    _os = require('os');

exports.configure = function(options) {
    if (_system.isNullOrUndefined(options)) {
        options = {};
    } else {
        options = _system.extend({}, options);
    }

    if (_system.isNullOrUndefined(options.indent)) {
        options.indent = '    ';
    }

    if (_system.isNullOrUndefined(options.EOL)) {
        options.EOL = _os.EOL;
    }

    return options;
};

function parseSwitchParameter(commandSwitch, args, index, result, parameterName) {
    var data = [];
    index = parseSwitchParameters(commandSwitch, args, index, data, 1);
    result[parameterName] = data[0];
    return index;
}

function parseSwitchParameters(commandSwitch, args, index, result, maxParameterCount) {
    var count = 0;
    if (index === args.length) {
        throw new Error("No value specified for command switch {0}.", commandSwitch);
    }

    while (index < args.length && args[index].substring(0, 2) != '--') {
        if (maxParameterCount && ++count > maxParameterCount) {
            throw new Error("Invalid number of parameters for command switch '{0}'.", commandSwitch);
        }
        result.push(args[index++]);
    }

    return index;
}

exports.getOption = function(sources, option, defaultValue) {
    sources = Array.isArray(sources) ? sources : [sources];
    for (var i= 0, n=sources.length; i< n; i++) {
        if (option in sources[i]) {
            return sources[i][option];
        }
    }
    return defaultValue;
}

exports.parse = function (args) {
    args = args || process.argv.slice(2);
    var options = {
            modules:[],
            monitor:false
        },
        i = 0, n = args.length;

    while (i < n) {
        var commandSwitch = args[i++];
        switch (commandSwitch) {
            case '--jsm':
                i = parseSwitchParameters(commandSwitch, args, i, options.modules);
                break;
            case '--formats':
                i = parseSwitchParameters(commandSwitch, args, i, options.formats = []);
                break;
            case '--monitor':
                options.monitor = true;
                break;
            case '--add-source-map-directive':
                options.addSourceMapDirective = true;
                break;
            case '--configuration':
                i = parseSwitchParameter(commandSwitch, args, i, options, 'configuration');
                break;
            case '--error-strategy':
                i = parseSwitchParameter(commandSwitch, args, i, options, 'errorStrategy');
                break;
            case '--version':
                i = parseSwitchParameter(commandSwitch, args, i, options, 'version');
                break;
            default:
                throw new Error(_system.string.format("Invalid command switch '{0}'.", commandSwitch))
        }
    }

    if (options.modules.length === 0) {
        throw new Error("No module files specified, use the --jsm switch to specify one or more module files.");
    }

    return options;
};